# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Call this with the font file as the command line argument.

import os
import sys

# Add paths so this runs in CPython in-place.
sys.path.append(os.path.join(sys.path[0], ".."))
from adafruit_bitmap_font import bitmap_font  # pylint: disable=wrong-import-position

sys.path.append(os.path.join(sys.path[0], "../test"))
font = bitmap_font.load_font(sys.argv[1])
specimen = "Adafruit CircuitPython" if len(sys.argv) == 2 else sys.argv[2]

_, height, _, dy = font.get_bounding_box()
font.load_glyphs(specimen)

for y in range(height):
    for c in specimen:
        glyph = font.get_glyph(ord(c))
        if not glyph:
            continue
        glyph_y = y + (glyph.height - (height + dy)) + glyph.dy
        pixels = []
        if 0 <= glyph_y < glyph.height:
            for i in range(glyph.width):
                value = glyph.bitmap[i, glyph_y]
                pixel = " "
                if value > 0:
                    pixel = "#"
                pixels.append(pixel)
        else:
            pixels = ""
        print("".join(pixels) + " " * (glyph.shift_x - len(pixels)), end="")
    print()



# SPDX-FileCopyrightText: 2019 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bitmap_font.bitmap_font`
====================================================

Loads bitmap glyphs from a variety of font.

* Author(s): Scott Shawcroft

Implementation Notes
--------------------

**Hardware:**

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases

"""

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Bitmap_Font.git"


def load_font(filename, bitmap=None):
    """Loads a font file. Returns None if unsupported."""
    # pylint: disable=import-outside-toplevel
    if not bitmap:
        import displayio

        bitmap = displayio.Bitmap
    font_file = open(filename, "rb")
    first_four = font_file.read(4)
    if filename.endswith("bdf") and first_four == b"STAR":
        from . import bdf

        return bdf.BDF(font_file, bitmap)
    if filename.endswith("pcf") and first_four == b"\x01fcp":
        from . import pcf

        return pcf.PCF(font_file, bitmap)
    if filename.endswith("ttf") and first_four == b"\x00\x01\x00\x00":
        from . import ttf

        return ttf.TTF(font_file, bitmap)

    raise ValueError("Unknown magic number %r" % first_four)



# SPDX-FileCopyrightText: 2021 Tim Cocks
# SPDX-License-Identifier: MIT

"""
This example uses addfruit_display_text.label to display fork awesome
icons.

More info here: https://emergent.unpythonic.net/01606790241
"""

import board
from forkawesome_icons import microchip, python, terminal
from adafruit_display_text import label
from adafruit_bitmap_font import bitmap_font

# use built in display (PyPortal, PyGamer, PyBadge, CLUE, etc.)
# see guide for setting up external displays (TFT / OLED breakouts, RGB matrices, etc.)
# https://learn.adafruit.com/circuitpython-display-support-using-displayio/display-and-display-bus
display = board.DISPLAY

font_file = "fonts/forkawesome-42.pcf"

# Set text, font, and color
text = "{}  {}  {}".format(terminal, python, microchip)
font = bitmap_font.load_font(font_file)
color = 0xFF00FF

# Create the tet label
text_area = label.Label(font, text=text, color=color)

# Set the location
text_area.anchor_point = (0.5, 0.5)
text_area.anchored_position = (display.width // 2, display.height // 2)

# Show it
display.show(text_area)

while True:
    pass



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

"""
This example uses addfruit_display_text.label to display text using a custom font
loaded by adafruit_bitmap_font
"""

import board
from adafruit_display_text import label
from adafruit_bitmap_font import bitmap_font

# use built in display (PyPortal, PyGamer, PyBadge, CLUE, etc.)
# see guide for setting up external displays (TFT / OLED breakouts, RGB matrices, etc.)
# https://learn.adafruit.com/circuitpython-display-support-using-displayio/display-and-display-bus
display = board.DISPLAY

# try uncommenting different font files if you like
font_file = "fonts/LeagueSpartan-Bold-16.bdf"
# font_file = "fonts/Junction-regular-24.pcf"

# Set text, font, and color
text = "HELLO WORLD"
font = bitmap_font.load_font(font_file)
color = 0xFF00FF

# Create the tet label
text_area = label.Label(font, text=text, color=color)

# Set the location
text_area.x = 20
text_area.y = 20

# Show it
display.show(text_area)

while True:
    pass



# SPDX-FileCopyrightText: 2020 Jeff Epler
# SPDX-License-Identifier: MIT

"""
Helper for fork awesome icon names.

More info here: https://emergent.unpythonic.net/01606790241
"""
activitypub = "\uf2f2"
address_book_o = "\uf2ba"
address_book = "\uf2b9"
address_card_o = "\uf2bc"
address_card = "\uf2bb"
adjust = "\uf042"
adn = "\uf170"
align_center = "\uf037"
align_justify = "\uf039"
align_left = "\uf036"
align_right = "\uf038"
amazon = "\uf270"
ambulance = "\uf0f9"
american_sign_language_interpreting = "\uf2a3"
anchor = "\uf13d"
android = "\uf17b"
angellist = "\uf209"
angle_double_down = "\uf103"
angle_double_left = "\uf100"
angle_double_right = "\uf101"
angle_double_up = "\uf102"
angle_down = "\uf107"
angle_left = "\uf104"
angle_right = "\uf105"
angle_up = "\uf106"
apple = "\uf179"
archive_org = "\uf2fc"
archive = "\uf187"
archlinux = "\uf323"
area_chart = "\uf1fe"
arrow_circle_down = "\uf0ab"
arrow_circle_left = "\uf0a8"
arrow_circle_o_down = "\uf01a"
arrow_circle_o_left = "\uf190"
arrow_circle_o_right = "\uf18e"
arrow_circle_o_up = "\uf01b"
arrow_circle_right = "\uf0a9"
arrow_circle_up = "\uf0aa"
arrow_down = "\uf063"
arrow_left = "\uf060"
arrow_right = "\uf061"
arrows_alt = "\uf0b2"
arrows_h = "\uf07e"
arrows = "\uf047"
arrows_v = "\uf07d"
arrow_up = "\uf062"
artstation = "\uf2ed"
askfm = "\uf33a"
assistive_listening_systems = "\uf2a2"
asterisk = "\uf069"
att = "\uf31e"
at = "\uf1fa"
audio_description = "\uf29e"
backward = "\uf04a"
balance_scale = "\uf24e"
bandcamp = "\uf2d5"
ban = "\uf05e"
bar_chart = "\uf080"
barcode = "\uf02a"
bars = "\uf0c9"
bath = "\uf2cd"
battery_empty = "\uf244"
battery_full = "\uf240"
battery_half = "\uf242"
battery_quarter = "\uf243"
battery_three_quarters = "\uf241"
bed = "\uf236"
beer = "\uf0fc"
behance_square = "\uf1b5"
behance = "\uf1b4"
bell_o = "\uf0f3"
bell_ringing_o = "\uf330"
bell_ringing = "\uf32d"
bell_slash_o = "\uf1f7"
bell_slash = "\uf1f6"
bell = "\uf0a2"
bicycle = "\uf206"
binoculars = "\uf1e5"
biometric = "\uf32b"
birthday_cake = "\uf1fd"
bitbucket_square = "\uf172"
bitbucket = "\uf171"
black_tie = "\uf27e"
blind = "\uf29d"
blockstack = "\uf33b"
bluetooth_b = "\uf294"
bluetooth = "\uf293"
boardgamegeek = "\uf33c"
bold = "\uf032"
bolt = "\uf0e7"
bomb = "\uf1e2"
bookmark_o = "\uf097"
bookmark = "\uf02e"
book = "\uf02d"
bootstrap = "\uf315"
braille = "\uf2a1"
briefcase = "\uf0b1"
btc = "\uf15a"
bug = "\uf188"
building_o = "\uf0f7"
building = "\uf1ad"
bullhorn = "\uf0a1"
bullseye = "\uf140"
bunny = "\uf35f"
bus = "\uf207"
buymeacoffee = "\uf33d"
buysellads = "\uf20d"
calculator = "\uf1ec"
calendar_check_o = "\uf274"
calendar_minus_o = "\uf272"
calendar_o = "\uf133"
calendar_plus_o = "\uf271"
calendar_times_o = "\uf273"
calendar = "\uf073"
camera_retro = "\uf083"
camera = "\uf030"
caret_down = "\uf0d7"
caret_left = "\uf0d9"
caret_right = "\uf0da"
caret_square_o_down = "\uf150"
caret_square_o_left = "\uf191"
caret_square_o_right = "\uf152"
caret_square_o_up = "\uf151"
caret_up = "\uf0d8"
cart_arrow_down = "\uf218"
cart_plus = "\uf217"
car = "\uf1b9"
cc_amex = "\uf1f3"
cc_by = "\uf33e"
cc_cc = "\uf33f"
cc_diners_club = "\uf24c"
cc_discover = "\uf1f2"
cc_jcb = "\uf24b"
cc_mastercard = "\uf1f1"
cc_nc_eu = "\uf341"
cc_nc_jp = "\uf342"
cc_nc = "\uf340"
cc_nd = "\uf343"
cc_paypal = "\uf1f4"
cc_pd = "\uf344"
cc_remix = "\uf345"
cc_sa = "\uf346"
cc_share = "\uf347"
cc_stripe = "\uf1f5"
cc = "\uf20a"
cc_visa = "\uf1f0"
cc_zero = "\uf348"
certificate = "\uf0a3"
chain_broken = "\uf127"
check_circle_o = "\uf05d"
check_circle = "\uf058"
check_square_o = "\uf046"
check_square = "\uf14a"
check = "\uf00c"
chevron_circle_down = "\uf13a"
chevron_circle_left = "\uf137"
chevron_circle_right = "\uf138"
chevron_circle_up = "\uf139"
chevron_down = "\uf078"
chevron_left = "\uf053"
chevron_right = "\uf054"
chevron_up = "\uf077"
child = "\uf1ae"
chrome = "\uf268"
circle_o_notch = "\uf1ce"
circle_o = "\uf10c"
circle_thin = "\uf1db"
circle = "\uf111"
classicpress_circle = "\uf332"
classicpress = "\uf331"
clipboard = "\uf0ea"
clock_o = "\uf017"
clone = "\uf24d"
cloud_download = "\uf0ed"
cloud = "\uf0c2"
cloud_upload = "\uf0ee"
code_fork = "\uf126"
codepen = "\uf1cb"
code = "\uf121"
codiepie = "\uf284"
coffee = "\uf0f4"
cogs = "\uf085"
cog = "\uf013"
columns = "\uf0db"
commenting_o = "\uf27b"
commenting = "\uf27a"
comment_o = "\uf0e5"
comments_o = "\uf0e6"
comments = "\uf086"
comment = "\uf075"
compass = "\uf14e"
compress = "\uf066"
connectdevelop = "\uf20e"
contao = "\uf26d"
conway_glider = "\uf349"
copyright = "\uf1f9"
creative_commons = "\uf25e"
credit_card_alt = "\uf283"
credit_card = "\uf09d"
crop = "\uf125"
crosshairs = "\uf05b"
csharp = "\uf34a"
css3 = "\uf13c"
cubes = "\uf1b3"
cube = "\uf1b2"
c = "\uf31c"
cutlery = "\uf0f5"
dashcube = "\uf210"
database = "\uf1c0"
deaf = "\uf2a4"
debian = "\uf2ff"
delicious = "\uf1a5"
desktop = "\uf108"
deviantart = "\uf1bd"
dev_to = "\uf316"
diamond = "\uf219"
diaspora = "\uf2e5"
digg = "\uf1a6"
digitalocean = "\uf31d"
discord_alt = "\uf2ef"
discord = "\uf2ee"
dogmazic = "\uf303"
dot_circle_o = "\uf192"
download = "\uf019"
dribbble = "\uf17d"
dropbox = "\uf16b"
drupal = "\uf1a9"
edge = "\uf282"
eercast = "\uf2da"
eject = "\uf052"
ellipsis_h = "\uf141"
ellipsis_v = "\uf142"
email_bulk_o = "\uf34c"
email_bulk = "\uf34b"
emby = "\uf319"
empire = "\uf1d1"
envelope_open_o = "\uf2b7"
envelope_open = "\uf2b6"
envelope_o = "\uf003"
envelope_square = "\uf199"
envelope = "\uf0e0"
envira = "\uf299"
eraser = "\uf12d"
ethereum = "\uf2f3"
etsy = "\uf2d7"
eur = "\uf153"
exchange = "\uf0ec"
exclamation_circle = "\uf06a"
exclamation_triangle = "\uf071"
exclamation = "\uf12a"
expand = "\uf065"
expeditedssl = "\uf23e"
external_link_square = "\uf14c"
external_link = "\uf08e"
eyedropper = "\uf1fb"
eye_slash = "\uf070"
eye = "\uf06e"
facebook_messenger = "\uf2fe"
facebook_official = "\uf230"
facebook_square = "\uf082"
facebook = "\uf09a"
fast_backward = "\uf049"
fast_forward = "\uf050"
fax = "\uf1ac"
f_droid = "\uf32a"
female = "\uf182"
ffmpeg = "\uf30f"
fighter_jet = "\uf0fb"
file_archive_o = "\uf1c6"
file_audio_o = "\uf1c7"
file_code_o = "\uf1c9"
file_epub = "\uf321"
file_excel_o = "\uf1c3"
file_image_o = "\uf1c5"
file_o = "\uf016"
file_pdf_o = "\uf1c1"
file_powerpoint_o = "\uf1c4"
files_o = "\uf0c5"
file_text_o = "\uf0f6"
file_text = "\uf15c"
file_ = "\uf15b"
file_video_o = "\uf1c8"
file_word_o = "\uf1c2"
film = "\uf008"
filter_ = "\uf0b0"
fire_extinguisher = "\uf134"
firefox = "\uf269"
fire = "\uf06d"
first_order = "\uf2b0"
fivehundredpx = "\uf26e"
flag_checkered = "\uf11e"
flag_o = "\uf11d"
flag = "\uf024"
flask = "\uf0c3"
flickr = "\uf16e"
floppy_o = "\uf0c7"
folder_open_o = "\uf115"
folder_open = "\uf07c"
folder_o = "\uf114"
folder = "\uf07b"
font_awesome = "\uf2b4"
fonticons = "\uf280"
font = "\uf031"
fork_awesome = "\uf2e3"
fort_awesome = "\uf286"
forumbee = "\uf211"
forward = "\uf04e"
foursquare = "\uf180"
free_code_camp = "\uf2c5"
freedombox = "\uf2fd"
friendica = "\uf2e6"
frown_o = "\uf119"
funkwhale = "\uf339"
futbol_o = "\uf1e3"
gamepad = "\uf11b"
gavel = "\uf0e3"
gbp = "\uf154"
genderless = "\uf22d"
get_pocket = "\uf265"
gg_circle = "\uf261"
gg = "\uf260"
gift = "\uf06b"
gimp = "\uf31b"
gitea = "\uf31f"
github_alt = "\uf113"
github_square = "\uf092"
github = "\uf09b"
gitlab = "\uf296"
git_square = "\uf1d2"
git = "\uf1d3"
glass = "\uf000"
glide_g = "\uf2a6"
glide = "\uf2a5"
globe_e = "\uf304"
globe = "\uf0ac"
globe_w = "\uf305"
gnupg = "\uf30d"
gnu_social = "\uf2e7"
gnu = "\uf34d"
google_play = "\uf34e"
google_plus_official = "\uf2b3"
google_plus_square = "\uf0d4"
google_plus = "\uf0d5"
google = "\uf1a0"
google_wallet = "\uf1ee"
graduation_cap = "\uf19d"
gratipay = "\uf184"
grav = "\uf2d6"
hackaday = "\uf30a"
hacker_news = "\uf1d4"
hackster = "\uf326"
hal = "\uf333"
hand_lizard_o = "\uf258"
hand_o_down = "\uf0a7"
hand_o_left = "\uf0a5"
hand_o_right = "\uf0a4"
hand_o_up = "\uf0a6"
hand_paper_o = "\uf256"
hand_peace_o = "\uf25b"
hand_pointer_o = "\uf25a"
hand_rock_o = "\uf255"
hand_scissors_o = "\uf257"
handshake_o = "\uf2b5"
hand_spock_o = "\uf259"
hashnode = "\uf317"
hashtag = "\uf292"
hdd_o = "\uf0a0"
header = "\uf1dc"
headphones = "\uf025"
heartbeat = "\uf21e"
heart_o = "\uf08a"
heart = "\uf004"
heroku = "\uf34f"
history = "\uf1da"
home_assistant = "\uf350"
home = "\uf015"
hospital_o = "\uf0f8"
hourglass_end = "\uf253"
hourglass_half = "\uf252"
hourglass_o = "\uf250"
hourglass_start = "\uf251"
hourglass = "\uf254"
houzz = "\uf27c"
h_square = "\uf0fd"
html5 = "\uf13b"
hubzilla = "\uf2eb"
i_cursor = "\uf246"
id_badge = "\uf2c1"
id_card_o = "\uf2c3"
id_card = "\uf2c2"
ils = "\uf20b"
imdb = "\uf2d8"
inbox = "\uf01c"
indent = "\uf03c"
industry = "\uf275"
info_circle = "\uf05a"
info = "\uf129"
inkscape = "\uf312"
inr = "\uf156"
instagram = "\uf16d"
internet_explorer = "\uf26b"
ioxhost = "\uf208"
italic = "\uf033"
java = "\uf351"
jirafeau = "\uf318"
joomla = "\uf1aa"
joplin = "\uf310"
jpy = "\uf157"
jsfiddle = "\uf1cc"
julia = "\uf334"
jupyter = "\uf335"
keybase = "\uf2f4"
keyboard_o = "\uf11c"
key_modern = "\uf2f7"
key = "\uf084"
krw = "\uf159"
language = "\uf1ab"
laptop = "\uf109"
laravel = "\uf30b"
lastfm_square = "\uf203"
lastfm = "\uf202"
leaf = "\uf06c"
leanpub = "\uf212"
lemon_o = "\uf094"
level_down = "\uf149"
level_up = "\uf148"
liberapay_square = "\uf2e8"
liberapay = "\uf2e9"
life_ring = "\uf1cd"
lightbulb_o = "\uf0eb"
line_chart = "\uf201"
linkedin_square = "\uf08c"
linkedin = "\uf0e1"
link = "\uf0c1"
linode = "\uf2b8"
linux = "\uf17c"
list_alt = "\uf022"
list_ol = "\uf0cb"
list_ = "\uf03a"
list_ul = "\uf0ca"
location_arrow = "\uf124"
lock = "\uf023"
long_arrow_down = "\uf175"
long_arrow_left = "\uf177"
long_arrow_right = "\uf178"
long_arrow_up = "\uf176"
low_vision = "\uf2a8"
magic = "\uf0d0"
magnet = "\uf076"
male = "\uf183"
map_marker = "\uf041"
map_o = "\uf278"
map_pin = "\uf276"
map_signs = "\uf277"
map_ = "\uf279"
mariadb = "\uf352"
markdown = "\uf353"
mars_double = "\uf227"
mars_stroke_h = "\uf22b"
mars_stroke = "\uf229"
mars_stroke_v = "\uf22a"
mars = "\uf222"
mastodon_alt = "\uf2e2"
mastodon_square = "\uf300"
mastodon = "\uf2e1"
matrix_org = "\uf313"
maxcdn = "\uf136"
meanpath = "\uf20c"
medium_square = "\uf2f8"
medium = "\uf23a"
medkit = "\uf0fa"
meetup = "\uf2e0"
meh_o = "\uf11a"
mercury = "\uf223"
microchip = "\uf2db"
microphone_slash = "\uf131"
microphone = "\uf130"
minus_circle = "\uf056"
minus_square_o = "\uf147"
minus_square = "\uf146"
minus = "\uf068"
mixcloud = "\uf289"
mobile = "\uf10b"
modx = "\uf285"
money = "\uf0d6"
moon_o = "\uf186"
moon = "\uf328"
motorcycle = "\uf21c"
mouse_pointer = "\uf245"
music = "\uf001"
mysql = "\uf354"
neuter = "\uf22c"
newspaper_o = "\uf1ea"
nextcloud_square = "\uf307"
nextcloud = "\uf306"
nodejs = "\uf308"
nordcast = "\uf355"
object_group = "\uf247"
object_ungroup = "\uf248"
odnoklassniki_square = "\uf264"
odnoklassniki = "\uf263"
opencart = "\uf23d"
open_collective = "\uf336"
openid = "\uf19b"
opera = "\uf26a"
optin_monster = "\uf23c"
orcid = "\uf337"
outdent = "\uf03b"
pagelines = "\uf18c"
paint_brush = "\uf1fc"
paperclip = "\uf0c6"
paper_plane_o = "\uf1d9"
paper_plane = "\uf1d8"
paragraph = "\uf1dd"
patreon = "\uf2f0"
pause_circle_o = "\uf28c"
pause_circle = "\uf28b"
pause = "\uf04c"
paw = "\uf1b0"
paypal = "\uf1ed"
peertube = "\uf2e4"
pencil_square_o = "\uf044"
pencil_square = "\uf14b"
pencil = "\uf040"
percent = "\uf295"
phone_square = "\uf098"
phone = "\uf095"
php = "\uf30e"
picture_o = "\uf03e"
pie_chart = "\uf200"
pinterest_p = "\uf231"
pinterest_square = "\uf0d3"
pinterest = "\uf0d2"
pixelfed = "\uf314"
plane = "\uf072"
play_circle_o = "\uf01d"
play_circle = "\uf144"
play = "\uf04b"
pleroma = "\uf324"
plug = "\uf1e6"
plume = "\uf356"
plus_circle = "\uf055"
plus_square_o = "\uf196"
plus_square = "\uf0fe"
plus = "\uf067"
podcast = "\uf2ce"
postgresql = "\uf357"
power_off = "\uf011"
print_ = "\uf02f"
product_hunt = "\uf288"
puzzle_piece = "\uf12e"
python = "\uf322"
qq = "\uf1d6"
qrcode = "\uf029"
question_circle_o = "\uf29c"
question_circle = "\uf059"
question = "\uf128"
quora = "\uf2c4"
quote_left = "\uf10d"
quote_right = "\uf10e"
random = "\uf074"
ravelry = "\uf2d9"
react = "\uf302"
rebel = "\uf1d0"
recycle = "\uf1b8"
reddit_alien = "\uf281"
reddit_square = "\uf1a2"
reddit = "\uf1a1"
refresh = "\uf021"
registered = "\uf25d"
renren = "\uf18b"
repeat = "\uf01e"
reply_all = "\uf122"
reply = "\uf112"
researchgate = "\uf338"
retweet = "\uf079"
road = "\uf018"
rocket = "\uf135"
rss_square = "\uf143"
rss = "\uf09e"
rub = "\uf158"
safari = "\uf267"
sass_alt = "\uf359"
sass = "\uf358"
scissors = "\uf0c4"
scribd = "\uf28a"
scuttlebutt = "\uf2ea"
search_minus = "\uf010"
search_plus = "\uf00e"
search = "\uf002"
sellsy = "\uf213"
server = "\uf233"
shaarli_o = "\uf2f6"
shaarli = "\uf2f5"
share_alt_square = "\uf1e1"
share_alt = "\uf1e0"
share_square_o = "\uf045"
share_square = "\uf14d"
share = "\uf064"
shield = "\uf132"
ship = "\uf21a"
shirtsinbulk = "\uf214"
shopping_bag = "\uf290"
shopping_basket = "\uf291"
shopping_cart = "\uf07a"
shower = "\uf2cc"
signalapp = "\uf30c"
signal = "\uf012"
sign_in = "\uf090"
sign_language = "\uf2a7"
sign_out = "\uf08b"
simplybuilt = "\uf215"
sitemap = "\uf0e8"
skate = "\uf35a"
sketchfab = "\uf35b"
skyatlas = "\uf216"
skype = "\uf17e"
slack = "\uf198"
sliders = "\uf1de"
slideshare = "\uf1e7"
smile_o = "\uf118"
snapchat_ghost = "\uf2ac"
snapchat_square = "\uf2ad"
snapchat = "\uf2ab"
snowdrift = "\uf2f1"
snowflake_o = "\uf2dc"
social_home = "\uf2ec"
sort_alpha_asc = "\uf15d"
sort_alpha_desc = "\uf15e"
sort_amount_asc = "\uf160"
sort_amount_desc = "\uf161"
sort_asc = "\uf0de"
sort_desc = "\uf0dd"
sort_numeric_asc = "\uf162"
sort_numeric_desc = "\uf163"
sort = "\uf0dc"
soundcloud = "\uf1be"
space_shuttle = "\uf197"
spell_check = "\uf327"
spinner = "\uf110"
spoon = "\uf1b1"
spotify = "\uf1bc"
square_o = "\uf096"
square = "\uf0c8"
stack_exchange = "\uf18d"
stack_overflow = "\uf16c"
star_half_o = "\uf123"
star_half = "\uf089"
star_o = "\uf006"
star = "\uf005"
steam_square = "\uf1b7"
steam = "\uf1b6"
step_backward = "\uf048"
step_forward = "\uf051"
stethoscope = "\uf0f1"
sticky_note_o = "\uf24a"
sticky_note = "\uf249"
stop_circle_o = "\uf28e"
stop_circle = "\uf28d"
stop = "\uf04d"
street_view = "\uf21d"
strikethrough = "\uf0cc"
stumbleupon_circle = "\uf1a3"
stumbleupon = "\uf1a4"
subscript = "\uf12c"
subway = "\uf239"
suitcase = "\uf0f2"
sun_o = "\uf185"
sun = "\uf329"
superpowers = "\uf2dd"
superscript = "\uf12b"
syncthing = "\uf311"
tablet = "\uf10a"
table = "\uf0ce"
tachometer = "\uf0e4"
tags = "\uf02c"
tag = "\uf02b"
tasks = "\uf0ae"
taxi = "\uf1ba"
telegram = "\uf2c6"
television = "\uf26c"
tencent_weibo = "\uf1d5"
terminal = "\uf120"
text_height = "\uf034"
textpattern = "\uf35d"
text_width = "\uf035"
tex = "\uf35c"
themeisle = "\uf2b2"
thermometer_empty = "\uf2cb"
thermometer_full = "\uf2c7"
thermometer_half = "\uf2c9"
thermometer_quarter = "\uf2ca"
thermometer_three_quarters = "\uf2c8"
th_large = "\uf009"
th_list = "\uf00b"
th = "\uf00a"
thumbs_down = "\uf165"
thumbs_o_down = "\uf088"
thumbs_o_up = "\uf087"
thumbs_up = "\uf164"
thumb_tack = "\uf08d"
ticket = "\uf145"
times_circle_o = "\uf05c"
times_circle = "\uf057"
times = "\uf00d"
tint = "\uf043"
tipeee = "\uf301"
toggle_off = "\uf204"
toggle_on = "\uf205"
tor_onion = "\uf32e"
trademark = "\uf25c"
train = "\uf238"
transgender_alt = "\uf225"
transgender = "\uf224"
trash_o = "\uf014"
trash = "\uf1f8"
tree = "\uf1bb"
trello = "\uf181"
tripadvisor = "\uf262"
trophy = "\uf091"
truck = "\uf0d1"
try_ = "\uf195"
tty = "\uf1e4"
tumblr_square = "\uf174"
tumblr = "\uf173"
twitch = "\uf1e8"
twitter_square = "\uf081"
twitter = "\uf099"
umbrella = "\uf0e9"
underline = "\uf0cd"
undo = "\uf0e2"
unity = "\uf35e"
universal_access = "\uf29a"
university = "\uf19c"
unlock_alt = "\uf13e"
unlock = "\uf09c"
unsplash = "\uf325"
upload = "\uf093"
usb = "\uf287"
usd = "\uf155"
user_circle_o = "\uf2be"
user_circle = "\uf2bd"
user_md = "\uf0f0"
user_o = "\uf2c0"
user_plus = "\uf234"
user_secret = "\uf21b"
users = "\uf0c0"
user_times = "\uf235"
user = "\uf007"
venus_double = "\uf226"
venus_mars = "\uf228"
venus = "\uf221"
viacoin = "\uf237"
viadeo_square = "\uf2aa"
viadeo = "\uf2a9"
video_camera = "\uf03d"
vimeo_square = "\uf194"
vimeo = "\uf27d"
vine = "\uf1ca"
vk = "\uf189"
volume_control_phone = "\uf2a0"
volume_down = "\uf027"
volume_mute = "\uf32f"
volume_off = "\uf026"
volume_up = "\uf028"
weibo = "\uf18a"
weixin = "\uf1d7"
whatsapp = "\uf232"
wheelchair_alt = "\uf29b"
wheelchair = "\uf193"
wifi = "\uf1eb"
wikidata = "\uf31a"
wikipedia_w = "\uf266"
window_close_o = "\uf2d4"
window_close = "\uf2d3"
window_maximize = "\uf2d0"
window_minimize = "\uf2d1"
window_restore = "\uf2d2"
windows = "\uf17a"
wire = "\uf32c"
wordpress = "\uf19a"
wpbeginner = "\uf297"
wpexplorer = "\uf2de"
wpforms = "\uf298"
wrench = "\uf0ad"
xing_square = "\uf169"
xing = "\uf168"
xmpp = "\uf2f9"
yahoo = "\uf19e"
y_combinator = "\uf23b"
yelp = "\uf1e9"
yoast = "\uf2b1"
youtube_play = "\uf16a"
youtube_square = "\uf166"
youtube = "\uf167"
zotero = "\uf309"



# SPDX-FileCopyrightText: 2019 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bitmap_font.glyph_cache`
====================================================

Displays text using CircuitPython's displayio.

* Author(s): Scott Shawcroft

Implementation Notes
--------------------

**Hardware:**

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases

"""

import gc

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Bitmap_Font.git"


class GlyphCache:
    """Caches glyphs loaded by a subclass."""

    def __init__(self):
        self._glyphs = {}

    def load_glyphs(self, code_points):
        """Loads displayio.Glyph objects into the GlyphCache from the font."""

    def get_glyph(self, code_point):
        """Returns a displayio.Glyph for the given code point or None is unsupported."""
        if code_point in self._glyphs:
            return self._glyphs[code_point]

        code_points = set()
        code_points.add(code_point)
        self._glyphs[code_point] = None
        self.load_glyphs(code_points)
        gc.collect()
        return self._glyphs[code_point]



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

"""
This example runs on PyPortal, or any Circuit Python device
with a built-in screen that is at least 320x240 pixels.

It will use adafruit_bitmap_font to load a font and fill a
bitmap with pixels matching glyphs from a given String
"""


import board
import displayio
from adafruit_bitmap_font import bitmap_font

# use built in display (PyPortal, PyGamer, PyBadge, CLUE, etc.)
# see guide for setting up external displays (TFT / OLED breakouts, RGB matrices, etc.)
# https://learn.adafruit.com/circuitpython-display-support-using-displayio/display-and-display-bus
display = board.DISPLAY

# try uncommenting different font files if you like
font_file = "fonts/LeagueSpartan-Bold-16.bdf"
# font_file = "fonts/Junction-regular-24.pcf"

font = bitmap_font.load_font(font_file)

bitmap = displayio.Bitmap(320, 240, 2)

palette = displayio.Palette(2)

palette[0] = 0x000000
palette[1] = 0xFFFFFF

_, height, _, dy = font.get_bounding_box()
for y in range(height):
    pixels = []
    for c in "Adafruit CircuitPython":
        glyph = font.get_glyph(ord(c))
        if not glyph:
            continue
        glyph_y = y + (glyph.height - (height + dy)) + glyph.dy

        if 0 <= glyph_y < glyph.height:
            for i in range(glyph.width):
                value = glyph.bitmap[i, glyph_y]
                pixel = 0
                if value > 0:
                    pixel = 1
                pixels.append(pixel)
        else:
            # empty section for this glyph
            for i in range(glyph.width):
                pixels.append(0)

        # one space between glyph
        pixels.append(0)

    if pixels:
        for x, pixel in enumerate(pixels):
            bitmap[x, y] = pixel

# Create a TileGrid to hold the bitmap
tile_grid = displayio.TileGrid(bitmap, pixel_shader=palette)

# Create a Group to hold the TileGrid
group = displayio.Group()

group.x = 20
# Add the TileGrid to the Group
group.append(tile_grid)

# Add the Group to the Display
display.show(group)

while True:
    pass



# SPDX-FileCopyrightText: 2019 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT

# pylint: skip-file
# Remove the above when TTF is actually supported.

import struct


# https://developer.apple.com/fonts/TrueType-Reference-Manual/RM06/Chap6glyf.html


class TTF:
    def __init__(self, f, bitmap):
        f.seek(0)
        self.file = f

        self.characters = {}

        def read(format):
            s = struct.calcsize(format)
            return struct.unpack_from(format, f.read(s))

        scalar_type = read(">I")
        numTables, searchRange, entrySelector, rangeShift = read(">HHHH")

        print(numTables)
        table_info = {}
        for _ in range(numTables):
            tag, checkSum, offset, length = read(">4sIII")
            print(tag.decode("utf-8"), hex(checkSum), offset, length)
            table_info[tag] = (offset, length)

        head_offset, head_length = table_info[b"head"]
        f.seek(head_offset)
        version, fontRevision, checkSumAdjustment, magicNumber = read(">IIII")
        flags, unitsPerEm, created, modified = read(">HHQQ")
        xMin, yMin, xMax, yMax = read(">hhhh")
        print(xMin, yMin, xMax, yMax)
        macStyle, lowestRecPPEM, fontDirectionHint = read(">HHh")
        indexToLocFormat, glyphDataFormat = read(">hh")

        glyf_offset, glyf_length = table_info[b"glyf"]
        f.seek(glyf_offset)
        while f.tell() < glyf_offset + glyf_length:
            numberOfContours, xMin, yMin, xMax, yMax = read(">hhhhh")

            if numberOfContours > 0:  # Simple
                print(numberOfContours)
                ends = []
                for _ in range(numberOfContours):
                    ends.append(read(">H"))
                instructionLength = read(">h")[0]
                instructions = read(">{}s".format(instructionLength))[0]
                print(instructions)
                break
            else:
                raise RuntimeError("Unsupported font")



# SPDX-FileCopyrightText: 2020 Jeff Epler for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bitmap_font.pcf`
====================================================

Loads PCF format fonts.

* Author(s): Jeff Epler

Implementation Notes
--------------------

**Hardware:**

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases

"""

from collections import namedtuple
import gc
import struct
from micropython import const

from fontio import Glyph
from .glyph_cache import GlyphCache

try:
    from bitmaptools import readinto as _bitmap_readinto
except ImportError:
    _bitmap_readinto = None  # pylint: disable=invalid-name

_PCF_PROPERTIES = const(1 << 0)
_PCF_ACCELERATORS = const(1 << 1)
_PCF_METRICS = const(1 << 2)
_PCF_BITMAPS = const(1 << 3)
_PCF_INK_METRICS = const(1 << 4)
_PCF_BDF_ENCODINGS = const(1 << 5)
_PCF_SWIDTHS = const(1 << 6)
_PCF_GLYPH_NAMES = const(1 << 7)
_PCF_BDF_ACCELERATORS = const(1 << 8)

_PCF_DEFAULT_FORMAT = const(0x00000000)
_PCF_ACCEL_W_INKBOUNDS = const(0x00000100)
_PCF_COMPRESSED_METRICS = const(0x00000100)

_PCF_GLYPH_PAD_MASK = const(3 << 0)  # See the bitmap table for explanation */
_PCF_BYTE_MASK = const(1 << 2)  # If set then Most Sig Byte First */
_PCF_BIT_MASK = const(1 << 3)  # If set then Most Sig Bit First */
_PCF_SCAN_UNIT_MASK = const(3 << 4)

# https://fontforge.org/docs/techref/pcf-format.html

Table = namedtuple("Table", ("format", "size", "offset"))
Metrics = namedtuple(
    "Metrics",
    (
        "left_side_bearing",
        "right_side_bearing",
        "character_width",
        "character_ascent",
        "character_descent",
        "character_attributes",
    ),
)
Accelerators = namedtuple(
    "Accelerators",
    (
        "no_overlap",
        "constant_metrics",
        "terminal_font",
        "constant_width",
        "ink_inside",
        "ink_metrics",
        "draw_direction",
        "font_ascent",
        "font_descent",
        "max_overlap",
        "minbounds",
        "maxbounds",
        "ink_minbounds",
        "ink_maxbounds",
    ),
)
Encoding = namedtuple(
    "Encoding", ("min_byte2", "max_byte2", "min_byte1", "max_byte1", "default_char")
)
Bitmap = namedtuple("Bitmap", ("glyph_count", "bitmap_sizes"))


class PCF(GlyphCache):
    """Loads glyphs from a PCF file in the given bitmap_class."""

    def __init__(self, f, bitmap_class):
        super().__init__()
        self.file = f
        self.name = f
        f.seek(0)
        self.buffer = bytearray(1)
        self.bitmap_class = bitmap_class
        _, table_count = self._read("<4sI")
        self.tables = {}
        for _ in range(table_count):
            type_, format_, size, offset = self._read("<IIII")
            self.tables[type_] = Table(format_, size, offset)

        bitmap_format = self.tables[_PCF_BITMAPS].format
        if bitmap_format != 0xE:
            raise NotImplementedError("Unsupported format %s" % bitmap_format)

        self._accel = self._read_accelerator_tables()
        self._encoding = self._read_encoding_table()
        self._bitmaps = self._read_bitmap_table()

        self._ascent = self._accel.font_ascent
        self._descent = self._accel.font_descent

        minbounds = self._accel.ink_minbounds
        maxbounds = self._accel.ink_maxbounds
        width = maxbounds.right_side_bearing - minbounds.left_side_bearing
        height = maxbounds.character_ascent + maxbounds.character_descent

        self._bounding_box = (
            width,
            height,
            minbounds.left_side_bearing,
            -maxbounds.character_descent,
        )

    @property
    def ascent(self):
        """The number of pixels above the baseline of a typical ascender"""
        return self._ascent

    @property
    def descent(self):
        """The number of pixels below the baseline of a typical descender"""
        return self._descent

    def get_bounding_box(self):
        """Return the maximum glyph size as a 4-tuple of: width, height, x_offset, y_offset"""
        return self._bounding_box

    def _read(self, format_):
        size = struct.calcsize(format_)
        if size != len(self.buffer):
            self.buffer = bytearray(size)
        self.file.readinto(self.buffer)
        return struct.unpack_from(format_, self.buffer)

    def _seek_table(self, table):
        self.file.seek(table.offset)
        (format_,) = self._read("<I")

        if format_ & _PCF_BYTE_MASK == 0:
            raise RuntimeError("Only big endian supported")

        return format_

    def _read_encoding_table(self):
        encoding = self.tables[_PCF_BDF_ENCODINGS]
        self._seek_table(encoding)

        return Encoding(*self._read(">hhhhh"))

    def _read_bitmap_table(self):
        bitmaps = self.tables[_PCF_BITMAPS]
        format_ = self._seek_table(bitmaps)

        (glyph_count,) = self._read(">I")
        self.file.seek(bitmaps.offset + 8 + 4 * glyph_count)
        bitmap_sizes = self._read(">4I")
        return Bitmap(glyph_count, bitmap_sizes[format_ & 3])

    def _read_metrics(self, compressed_metrics):
        if compressed_metrics:
            (
                left_side_bearing,
                right_side_bearing,
                character_width,
                character_ascent,
                character_descent,
            ) = self._read("5B")
            left_side_bearing -= 0x80
            right_side_bearing -= 0x80
            character_width -= 0x80
            character_ascent -= 0x80
            character_descent -= 0x80
            attributes = 0
        else:
            (
                left_side_bearing,
                right_side_bearing,
                character_width,
                character_ascent,
                character_descent,
                attributes,
            ) = self._read(">5hH")
        return Metrics(
            left_side_bearing,
            right_side_bearing,
            character_width,
            character_ascent,
            character_descent,
            attributes,
        )

    def _read_accelerator_tables(self):
        # pylint: disable=too-many-locals
        accelerators = self.tables.get(_PCF_BDF_ACCELERATORS)
        if not accelerators:
            accelerators = self.tables.get(_PCF_ACCELERATORS)
        if not accelerators:
            raise RuntimeError("Accelerator table missing")

        format_ = self._seek_table(accelerators)
        has_inkbounds = format_ & _PCF_ACCEL_W_INKBOUNDS

        (
            no_overlap,
            constant_metrics,
            terminal_font,
            constant_width,
            ink_inside,
            ink_metrics,
            draw_direction,
            _,
            font_ascent,
            font_descent,
            max_overlap,
        ) = self._read(">BBBBBBBBIII")
        minbounds = self._read_metrics(False)
        maxbounds = self._read_metrics(False)
        if has_inkbounds:
            ink_minbounds = self._read_metrics(False)
            ink_maxbounds = self._read_metrics(False)
        else:
            ink_minbounds = minbounds
            ink_maxbounds = maxbounds

        return Accelerators(
            no_overlap,
            constant_metrics,
            terminal_font,
            constant_width,
            ink_inside,
            ink_metrics,
            draw_direction,
            font_ascent,
            font_descent,
            max_overlap,
            minbounds,
            maxbounds,
            ink_minbounds,
            ink_maxbounds,
        )

    def _read_properties(self):
        property_table_offset = self.tables[_PCF_PROPERTIES]["offset"]
        self.file.seek(property_table_offset)
        (format_,) = self._read("<I")

        if format_ & _PCF_BYTE_MASK == 0:
            raise RuntimeError("Only big endian supported")
        (nprops,) = self._read(">I")
        self.file.seek(property_table_offset + 8 + 9 * nprops)

        pos = self.file.tell()
        if pos % 4 > 0:
            self.file.read(4 - pos % 4)
        (string_size,) = self._read(">I")

        strings = self.file.read(string_size)
        string_map = {}
        i = 0
        for value in strings.split(b"\x00"):
            string_map[i] = value
            i += len(value) + 1

        self.file.seek(property_table_offset + 8)
        for _ in range(nprops):
            name_offset, is_string_prop, value = self._read(">IBI")

            if is_string_prop:
                yield (string_map[name_offset], string_map[value])
            else:
                yield (string_map[name_offset], value)

    def load_glyphs(self, code_points):
        # pylint: disable=too-many-statements,too-many-branches,too-many-nested-blocks,too-many-locals
        if isinstance(code_points, int):
            code_points = (code_points,)
        elif isinstance(code_points, str):
            code_points = [ord(c) for c in code_points]

        code_points = sorted(
            c for c in code_points if self._glyphs.get(c, None) is None
        )
        if not code_points:
            return

        indices_offset = self.tables[_PCF_BDF_ENCODINGS].offset + 14
        bitmap_offset_offsets = self.tables[_PCF_BITMAPS].offset + 8
        first_bitmap_offset = self.tables[_PCF_BITMAPS].offset + 4 * (
            6 + self._bitmaps.glyph_count
        )
        metrics_compressed = self.tables[_PCF_METRICS].format & _PCF_COMPRESSED_METRICS
        first_metric_offset = self.tables[_PCF_METRICS].offset + (
            6 if metrics_compressed else 8
        )
        metrics_size = 5 if metrics_compressed else 12

        # These will each _tend to be_ forward reads in the file, at least
        # sometimes we'll benefit from oofatfs's 512 byte cache and avoid
        # excess reads
        indices = [None] * len(code_points)
        for i, code_point in enumerate(code_points):
            enc1 = (code_point >> 8) & 0xFF
            enc2 = code_point & 0xFF

            if enc1 < self._encoding.min_byte1 or enc1 > self._encoding.max_byte1:
                continue
            if enc2 < self._encoding.min_byte2 or enc2 > self._encoding.max_byte2:
                continue

            encoding_idx = (
                (enc1 - self._encoding.min_byte1)
                * (self._encoding.max_byte2 - self._encoding.min_byte2 + 1)
                + enc2
                - self._encoding.min_byte2
            )
            self.file.seek(indices_offset + 2 * encoding_idx)
            (glyph_idx,) = self._read(">H")
            if glyph_idx != 65535:
                indices[i] = glyph_idx

        all_metrics = [None] * len(code_points)
        for i, code_point in enumerate(code_points):
            index = indices[i]
            if index is None:
                continue
            self.file.seek(first_metric_offset + metrics_size * index)
            all_metrics[i] = self._read_metrics(metrics_compressed)
        bitmap_offsets = [None] * len(code_points)
        for i, code_point in enumerate(code_points):
            index = indices[i]
            if index is None:
                continue
            self.file.seek(bitmap_offset_offsets + 4 * index)
            (bitmap_offset,) = self._read(">I")
            bitmap_offsets[i] = bitmap_offset

        # Batch creation of glyphs and bitmaps so that we need only gc.collect
        # once
        gc.collect()
        bitmaps = [None] * len(code_points)
        for i in range(len(all_metrics)):  # pylint: disable=consider-using-enumerate
            metrics = all_metrics[i]
            if metrics is not None:
                width = metrics.right_side_bearing - metrics.left_side_bearing
                height = metrics.character_ascent + metrics.character_descent
                bitmap = bitmaps[i] = self.bitmap_class(width, height, 2)
                self._glyphs[code_points[i]] = Glyph(
                    bitmap,
                    0,
                    width,
                    height,
                    metrics.left_side_bearing,
                    -metrics.character_descent,
                    metrics.character_width,
                    0,
                )

        for i, code_point in enumerate(code_points):
            metrics = all_metrics[i]
            if metrics is None:
                continue
            self.file.seek(first_bitmap_offset + bitmap_offsets[i])
            width = metrics.right_side_bearing - metrics.left_side_bearing
            height = metrics.character_ascent + metrics.character_descent

            bitmap = bitmaps[i]

            if _bitmap_readinto:
                _bitmap_readinto(
                    bitmap,
                    self.file,
                    bits_per_pixel=1,
                    element_size=4,
                    reverse_pixels_in_element=True,
                )
            else:
                words_per_row = (width + 31) // 32
                buf = bytearray(4 * words_per_row)
                start = 0
                for _ in range(height):
                    self.file.readinto(buf)
                    for k in range(width):
                        if buf[k // 8] & (128 >> (k % 8)):
                            bitmap[start + k] = 1
                    start += width



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

"""
This example uses addfruit_display_text.label to display text using a custom font
loaded by adafruit_bitmap_font.
Adapted for use on MagTag
"""
import time
import board
from adafruit_display_text import label
from adafruit_bitmap_font import bitmap_font

# use built in display (PyPortal, PyGamer, PyBadge, CLUE, etc.)
# see guide for setting up external displays (TFT / OLED breakouts, RGB matrices, etc.)
# https://learn.adafruit.com/circuitpython-display-support-using-displayio/display-and-display-bus
display = board.DISPLAY
# wait until we can refresh the display
time.sleep(display.time_to_refresh)

# try uncommenting different font files if you like
font_file = "fonts/LeagueSpartan-Bold-16.bdf"
# font_file = "fonts/Junction-regular-24.pcf"

# Set text, font, and color
text = "HELLO WORLD\nbitmap_font example"
font = bitmap_font.load_font(font_file)
color = 0xFFFFFF
background_color = 0x999999

# Create the tet label
text_area = label.Label(
    font,
    text=text,
    color=color,
    background_color=background_color,
    padding_top=3,
    padding_bottom=3,
    padding_right=4,
    padding_left=4,
)
text_area.line_spacing = 1.0
# Set the location
text_area.x = 20
text_area.y = 20

# Show it and refresh
display.show(text_area)
display.refresh()
while True:
    pass



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""Implementation of minimal displayio subset for testing"""

# pylint: disable=all
class Bitmap:
    def __init__(self, width, height, color_count):
        self.width = width
        self.height = height
        if color_count > 255:
            raise ValueError("Cannot support that many colors")
        self.values = bytearray(width * height)

    def __setitem__(self, index, value):
        if isinstance(index, tuple):
            index = index[0] + index[1] * self.width
        self.values[index] = value

    def __getitem__(self, index):
        if isinstance(index, tuple):
            index = index[0] + index[1] * self.width
        return self.values[index]

    def __len__(self):
        return self.width * self.height



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""Implementation of minimal fontio subset for testing"""

import collections

Glyph = collections.namedtuple(
    "Glyph",
    ["bitmap", "tile_index", "width", "height", "dx", "dy", "shift_x", "shift_y"],
)
